<?php

namespace Drupal\merlinone\Plugin\EntityBrowser\Widget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Utility\Token;
use Drupal\entity_browser\WidgetBase;
use Drupal\entity_browser\WidgetValidationManager;
use Drupal\file\Entity\File;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Provides an Entity Browser widget that integrates with the MerlinOne
 * media library
 *
 * @EntityBrowserWidget(
 *   id = "merlinone_search",
 *   label = @Translation("MerlinOne Search"),
 *   description = @Translation("Add files from the MerlinOne library")
 * )
 */
class MerlinOneSearch extends WidgetBase {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;

  /**
   * Constructs widget plugin.
   *
   * @param array $configuration
   * @param string $plugin_id
   * @param mixed $plugin_definition
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * @param \Drupal\entity_browser\WidgetValidationManager $validation_manager
   * @param \Drupal\Core\Session\AccountInterface $current_user
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   * @param \Drupal\Core\Utility\Token $token
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EventDispatcherInterface $event_dispatcher, EntityTypeManagerInterface $entity_type_manager, WidgetValidationManager $validation_manager, AccountInterface $current_user, FileSystemInterface $file_system, Token $token) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $event_dispatcher, $entity_type_manager, $validation_manager);
    $this->currentUser = $current_user;
    $this->fileSystem = $file_system;
    $this->token = $token;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('event_dispatcher'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.entity_browser.widget_validation'),
      $container->get('current_user'),
      $container->get('file_system'),
      $container->get('token')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'upload_location' => 'public://merlinone/[date:custom:Y]-[date:custom:m]',
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function getForm(array &$original_form, FormStateInterface $form_state, array $additional_widget_parameters) {
    $form = parent::getForm($original_form, $form_state, $additional_widget_parameters);
    $config = \Drupal::config('merlinone.settings');

    // Build MX embed URL
    $mx_domain = $config->get('mx_domain');
    $mx_url = $mx_domain;
    if (substr($mx_url, -1, 1) !== '/') {
      $mx_url = $mx_url . '/';
    }
    $mx_url = $mx_url . 'mx?altprofile=cms&cmstype=drupal';

    // Embedded search
    $form['merlinone_search'] = [
      '#type' => 'html_tag',
      '#tag' => 'iframe',
      '#attributes' => [
        'src' => $mx_url,
        'class' => 'merlinone-search-iframe',
        'width' => '100%',
        'height' => 500,
        'frameborder' => 0,
        'style' => 'padding: 0; width: 1px !important; min-width: 100% !important; overflow: hidden !important;',
      ],
    ];

    // Attach library and pass domain settings
    $form['#attached']['library'][] = 'merlinone/entity_browser';
    $form['#attached']['drupalSettings']['merlinone']['entity_browser']['mx_host'] = parse_url($mx_domain)['host'];

    // Hold the response from the search
    $form['merlinone_response'] = [
      '#type' => 'hidden',
      '#default_value' => '',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareEntities(array $form, FormStateInterface $form_state) {
    $entities = [];

    // Prepare destination.
    $uploadDir = $this->getUploadLocation();
    if (!file_exists($uploadDir)) {
      $this->fileSystem->mkdir($uploadDir, NULL, TRUE);
    }

    if ($merlinResponse = $form_state->getValue('merlinone_response')) {
      $items = json_decode($merlinResponse);
      foreach ($items as $item) {

        $filename = $item->cobject205;

        // Destination is the upload directory and the filename
        $destination = file_stream_wrapper_uri_normalize($uploadDir . '/' . $filename);

        try {
          // Get the file data
          $response = \Drupal::httpClient()
            ->get($item->url);
          $data = (string) $response->getBody();

          // Save it to the filesystem
          $local = file_unmanaged_save_data($data, $destination, FILE_EXISTS_RENAME);
        }
        catch (RequestException $exception) {
          drupal_set_message(t('Failed to fetch file due to error "%error"', ['%error' => $exception->getMessage()]), 'error');
          continue;
        }
        if (!$local) {
          drupal_set_message(t('@remote could not be saved to @path.', ['@remote' => $item->url, '@path' => $destination]), 'error');
          continue;
        }

        $mime = $response->getHeader('Content-Type');
        $mime = count($mime) ? $mime[0] : \Drupal::service('file.mime_type.guesser')->guess($local);

        // Create a managed file entity
        $file = File::create([
          'filename' => $filename,
          'uri' => $local,
          'filemime' => $mime,
          'uid' => $this->currentUser->id(),
          'original_item' => $item, // Hold original item to get data out in extended classes
        ]);

        $entities[] = $file;
      }
    }

    // Keep reference to the prepared entities
    $form_state->set('merlinone_prepared_entities', $entities);

    return $entities;
  }

  /**
   * Gets upload location.
   *
   * @return string
   *   Destination folder URI.
   */
  protected function getUploadLocation() {
    return $this->token->replace($this->configuration['upload_location']);
  }

  /**
   * {@inheritdoc}
   */
  public function submit(array &$element, array &$form, FormStateInterface $form_state) {
    $prepared_entities = $form_state->get('merlinone_prepared_entities');

    foreach ($prepared_entities as $key => $prepared_entity) {
      /** @var File $entity */
      $entity = $prepared_entities[$key];
      $entity->setPermanent();
      $entity->save();
    }

    $this->selectEntities($prepared_entities, $form_state);
    $this->clearFormValues($element, $form_state);
  }

  /**
   * Clear values from Merlin response form element.
   *
   * @param array $element
   *   Upload form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   */
  protected function clearFormValues(array &$element, FormStateInterface $form_state) {
    // The hidden value and user input needs to be removed otherwise files will
    // re-download when the selection form is submitted.
    $form_state->setValueForElement($element['merlinone_response'], '');
    NestedArray::setValue($form_state->getUserInput(), $element['merlinone_response']['#parents'], '');
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $configuration = $this->configuration;

    $form['upload_location'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Upload location'),
      '#default_value' => $configuration['upload_location'],
    ];

    return $form;
  }
}
